/* src/add_info.c - Ajout d'informations en mmoire
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <time.h>
#include "main.h"

CVSINFO("$Id: add_info.c,v 1.6 2003/02/23 14:45:27 romexzf Exp $");

extern struct badchans *badchead;
extern aKill *killhead;
extern struct badnicks *badnhead;
extern struct ignore *ignorehead;

#ifdef HAVE_CRYPTHOST
struct nickinfo *add_nickinfo(char *nick, char *user, char *host, char *crypt, char *base64, char *num, char *server, char *name, long ttmco, char *modes)
#else
struct nickinfo *add_nickinfo(char *nick, char *user, char *host, char *base64, char *num, char *server, char *name, long ttmco, char *modes)
#endif
{
   struct nickinfo *new;
   new = (struct nickinfo *) calloc(1, sizeof(struct nickinfo));
   if(!new)
   {
	   putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aNick %s !\n", nick);
	   return NULL;
   }
   new->next = NULL;
   strncpy(new->nick, nick, NICKLEN);
   strncpy(new->ident, user, USERLEN);
   strncpy(new->host, host, HOSTLEN);

#ifdef HAVE_CRYPTHOST
   strncpy(new->crypt, strchr(modes, 'k') ? host : crypt, HOSTLEN);
#endif

   strcpy(new->base64, base64);
   strcpy(new->numeric, num);
   strcpy(new->server, server);
   strncpy(new->name, name, REALEN);
   if(modes[0]=='+') modes++;
   strncpy(new->modes, modes, 15);
   if(strchr(modes, 'o')) new->flag |= N_OPER;
   new->ttmco=ttmco;
   new->floodtime=time(NULL);
   new->floodcount=0;
   new->last=NULL;
   if(!nickhead) nickhead = new;
   else
   {
	   nickhead->last=new;
	   new->next=nickhead;
	   nickhead=new;
   }
   return new;
}

#ifdef USE_MEMOSERV
void add_memo(anUser *nick, char *de, int date, char *message)
{
	struct memos *new, *current;

	new = (struct memos *) calloc(1, sizeof(struct memos));
	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aMemo %s !\n", nick->nick);
		return;
	}
	new->next = NULL;
	strncpy(new->de, de, NICKLEN);
	new->date = date;
	strncpy(new->message, message, MEMOLEN);
	if(!nick->memohead) nick->memohead = new;
	else
	{
		for(current=nick->memohead;current->next;current=current->next);
		if(!current->next) current->next = new;
		else free(new);
	}
}
#endif

struct badchans *add_abadchan(char *chaine, char *raison)
{
	struct badchans *new, *current;
	new = (struct badchans *) calloc(1, sizeof(struct badchans));

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un badchans (%s) !\n", chaine);
		return NULL;
	}
	new->next = NULL;
	strcpy(new->chaine, chaine);
	strcpy(new->raison, raison);
	if(!badchead) badchead = new;
	else
	{
		for(current=badchead;current->next && match(current->chaine, new->chaine);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
	return new;
}

struct chaninfo *add_chan(char *chan, char *modes, char *topic, char *description, char *defmodes, char *deftopic, char *welcome, char *suspend, int flag, int banlevel, int bantype, char *motd, char *chanurl)
{
	aChan *new, *current;
	new = (struct chaninfo *) calloc(1, sizeof(struct chaninfo));

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aChan %s !\n", chan);
		return NULL;
	}
	new->next = NULL;
	strcpy(new->chan, chan);
	strcpy(new->modes, modes);
	strncpy(new->topic, topic,TOPICLEN);
	strcpy(new->defmodes, defmodes);
	strncpy(new->deftopic, deftopic,TOPICLEN);
	strncpy(new->description, description,150);
	strcpy(new->welcome, welcome);
	strncpy(new->suspend, suspend, 200);
	new->flag = flag;
	new->banlevel = banlevel;
	new->bantype = (bantype > 4 || bantype < 1) ? 1 : bantype;
	strncpy(new->motd, motd, 250);
	strncpy(new->url, chanurl, 100);
	new->banhead=NULL;

	if(!chanhead) chanhead = new;
	else
	{
		for(current=chanhead;current->next && strcasecmp(current->chan, chan);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
	return new;
}

void add_access(anUser *user, char *chan, int level, int flag, char *info, long lastseen)
{
	struct access *new, *current;
	aChan *c;
	aLink *lp;

	new = (struct access *) calloc(1, sizeof(struct access));
	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour anAccess %s sur %s!\n", user->nick, chan);
		return;
	}
	new->next = NULL;
	new->level = level;
	new->flag = flag;
	new->user = user;
	strncpy(new->info, info, 199);
	strcpy(new->chan, chan);
	if((c=getchaninfo(chan))) new->c = c;
	else
	{
		free(new);
		return;
	}

	if(lastseen) new->lastseen = lastseen;
	else new->lastseen = user->lastseend;

	if(!user->accesshead) user->accesshead = new;
	else
	{
		for(current=user->accesshead;current->next && strcasecmp(current->chan,chan);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}

	lp = (aLink *) calloc(1, sizeof(aLink));
	lp->value.a = new;
	lp->next = c->access;
	c->access = lp;
}

void add_killinfo(char *nick, char *numeric, long timestamp, int why, int setkill)
{
	struct killinfo *new, *current;
	new = (struct killinfo *) calloc(1, sizeof(struct killinfo));

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour killinfo %s !\n", nick);
		return;
	}
	new->next = NULL;
	strcpy(new->nick, nick);
	strcpy(new->numeric, numeric);
	new->timestamp = timestamp;
	new->why = why;
	new->setkill = setkill;
	if(!killhead) killhead = new;
	else
	{
		for(current=killhead;current->next;current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
}

void add_abadnick(char *chaine)
{
	struct badnicks *new, *current;
	new = (struct badnicks *) calloc(1, sizeof(struct badnicks));

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un badnick %s !\n", chaine);
		return;
	}
	new->next = NULL;
	strcpy(new->chaine, chaine);
	if(!badnhead) badnhead = new;
	else
	{
		for(current = badnhead;current->next && match(current->chaine, new->chaine);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
}

struct userinfo *add_regnick(char *user, char *pass, int lastseend, int level, int flag, int suspendu, char *mail, int vote)
{
	struct userinfo *new;
	new = (struct userinfo *) calloc(1,sizeof(struct userinfo));

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour anUser %s !\n", user);
		return NULL;
	}
	new->next = NULL;
	strcpy(new->nick, user);
	strcpy(new->passwd, pass);
	new->lastseend = lastseend;
	new->level = level;
	new->suspendu = suspendu;
	strcpy(new->mail, mail);
#ifdef HAVE_VOTE
	new->vote = vote;
#endif
	new->flag = flag;
	new->last=NULL;
	if(!userhead) userhead = new;
	else
	{
		userhead->last=new;
		new->next=userhead;
		userhead=new;
	}
	return new;
}

void add_join(aNick *nick, char *chan, int is_op, int is_voice, long timestamp, int is_burst)
{

	aJoin *new, *join;
	anAccess *access;
	aChan *chaninfo = NULL;
	aLink *lp, *l;
	struct baninfo *ban;
	char host[200];

	new = (struct joininfo *) calloc(1, sizeof(struct joininfo));

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aJoin %s sur %s !\n", nick->nick, chan);
		return;
	}
	new->next = NULL;
	strcpy(new->channel, chan);
	new->is_op = is_op;
	new->is_voice = is_voice;
	new->timestmp = timestamp;
	new->nick = nick;
	if(!nick->joinhead) nick->joinhead = new;
	else
	{
		for(join = nick->joinhead;join->next;join=join->next);
		if(!join->next && strcasecmp(join->channel, chan)) join->next=new;
		else /* le mec est dja sur le chan ? */
		{
			free(new);
			new=NULL;
		}
	}

	if(!(chaninfo = getchaninfo(chan))) return;
	if(new)
	{
		lp = (aLink *) calloc(1, sizeof(aLink));
		lp->value.j = new;
		if(!chaninfo->members) chaninfo->members = lp;
		else
		{
			for(l = chaninfo->members;l->next;l=l->next); /* on cherche le dernier */
			l->next = lp;
		}
	}

	if(chaninfo->suspend[0] != '\0') return;//chaninfo + reg? + suspend? (kouak)
	if(!is_burst)
	{
		//ban check CESARRRRRRRRRRR (mis ici pr eviter 2 loooooooops)
		if(!(nick->flag & N_OPER) && (!nick->user || !IsAnAdmin(nick->user)))
		{
			snprintf(host, (HOSTLEN + NICKLEN + USERLEN), "%s!%s@%s", nick->nick, nick->ident, nick->host);
			ban = chaninfo->banhead;
			while(ban)
			{
				if(!match(ban->mask, host))
				{
					csmode(chaninfo, "+b %s", ban->mask);
					cskick(chan, nick->numeric, "Enforce: %s!", ban->raison);
					return;
				}
				ban = ban->next;
			}
		}

		if(CSetWelcome(chaninfo) && chaninfo->welcome[0] != '\0') //si le welcome est activ ET que le welcome existe
		   csntc(nick, "Bienvenue sur \002%s\002 - %s", chaninfo->chan, chaninfo->welcome); //on envoie la notice
	}
	//On regarde l'access ici pour infoline, autoop et autovoice
	if(!nick->user || nick->user->suspendu) return; //ne va pas plus loin si l'user n'est pas auth ou qu'il est suspend (peut etre inutile)

	if(!(access=GetAccessIbyUserI(nick->user,chan)) || ASuspend(access)) return;//ne va pas plus loin si l'user n'a pas d'access ou qu'il est suspendu
	access->lastseen = 1;

	if(is_burst) return;

	if((AOp(access) || AVoice(access)) && access->level >= 50)
	{
		new->is_voice = AVoice(access);
		new->is_op = AOp(access);

		csmode(chaninfo, "+%s%s %s %s", AOp(access) ? (!CNoOps(chaninfo) ? "o" : ""):"", AVoice(access) ? "v" : "", AOp(access) ? (!CNoOps(chaninfo) ? nick->numeric : "") : "" , AVoice(access) ? nick->numeric : "");
	}

	if(access->info[0] != '\0')
	  putserv("%s %s %s :[\002%s\002] %s\n", cs.num, TOKEN_PRIVMSG, chan, nick->user->nick, access->info);
	return;

}

void add_ban(struct chaninfo *cchan, char *mask, char *raison, char *de, long debut, long fin, int level)
{
	struct baninfo *new, *current;

	new = (struct baninfo *) calloc(1, sizeof(struct baninfo));
	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aBan %s sur %s !\n", mask, cchan->chan);
		return;
	}
	strcpy(new->mask, mask);
	strncpy(new->raison, raison, 300);
	strncpy(new->de, de, NICKLEN);
	new->debut = debut;
	new->fin = fin;
	new->level = level;
	if(!cchan->banhead) cchan->banhead = new;
	else
	{
		for(current = cchan->banhead;current->next && match(current->mask, mask);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
}

void add_ignore(char *host, long expiration)
{
	struct ignore *new, *current;
	new = (struct ignore *) calloc(1, sizeof(struct ignore));

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un ignore %s !\n", host);
		return;
	}
	new->next = NULL;
	strncpy(new->host, host, HOSTLEN);
	new->exp = expiration;
	if(!ignorehead) ignorehead = new;
	else
	{
		for(current = ignorehead;current->next;current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
}

void add_waitaccess(struct userinfo *user, char *chan, int level)
{
	aWaitAccess *new, *current;

	new = (struct waitaccess *) calloc(1, sizeof(struct waitaccess));
	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aWaitAccess %s sur %s !\n", user->nick, chan);
		return;
	}
	new->next = NULL;
	new->level = level;
	strcpy(new->chan, chan);
	if(!user->wahead) user->wahead = new;
	else
	{
		for(current = user->wahead;current->next && strcasecmp(current->chan, chan);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
	return;
}
