/* src/main.c - Fichier principal
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <unistd.h>
#include <signal.h>
#include <time.h>
#define GLOBALS_DEF
#include "main.h"
#undef GLOBALS_DEF

CVSINFO("$Id: main.c,v 1.7 2003/03/13 16:31:58 progs Exp $");

int STS;
int lastresetparse = 0;
int lastresetlogs = 0;
int want_to_quit;
int lastcheckaccounts = 0;
struct userinfo *userhead = NULL;
struct nickinfo *nickhead = NULL;
struct badchans *badchead = NULL;
struct chaninfo *chanhead = NULL;
struct killinfo *killhead = NULL;
struct badnicks *badnhead = NULL;
struct ignore *ignorehead = NULL;
struct welcomeinfo *welcomehead = NULL;

/*
 *            FONCTIONS COMMANDES
 *            =-=-=-=-=-=-=-=-=-=
 *
 *    Il y a deux types de fonctions commandes.
 *    Les normales :
 *       parv[0] = pseudo du robot appel
 *       parv[1->parc] = arguments de la commandes
 *    Et les commandes salons (voir struct cs_commands) :
 *       parv[0] = pseudo du robot appel
 *       parv[1] = salon appel
 *       parv[2->parv] = arguments de la commande (aprs le nom du salon)
 *    Toute sles commandes doivent respecter ce shema.
 *    Lorsqu'une commande salon est tape sur un salon, et que le premier
 *    argument n'est pas un nom de salon, le nom du salon actuel est plac
 *    automatiquement comme premier argument.
 *
 *    Pour faciliter la lecture, et au cas o il pouvait y avoir plusieurs cas
 *    (ex: cmd op, if (parc == 0) cible = nick->nick else cible = parv[1]).
 *
 *    Chaque commande emploie une entte particulire :
 *    void commande(aNick *nick, aChan *chaninfo, int parc, char **parv)
 *      aNick *nick = structure nickinfo de l'user ki a appel la fonction
 *      aChan *chaninfo = structure chaninfo du salon si la cmd est chan=1 (sinon chaninfo=NULL)
 *      int parc = nombre d'arguments *SANS COMPTER PARV[0]*. (si le dernier arg
 *                 est parv[5], alors parc = 5.)
 *      char **parv = tableau contenant les arguments (une ligne = un mot).
 *
 *    Outils utiles :
 *      parv2msgn(int parc, char **parv, int position, char buf, int size)
 *          permet de convertire une fin de tableau en un message normal.
 *          il faut spcifier le tableau et le nombre d'arguments contenu
 *          ainsi que la position  laquelle on va commencer  enregistrer
 *          le contenu dans la variable envoye en paramtre (taille max size).
 *      nick2numinfo(char nick), num2nickinfo(char num)
 *          renvoit la struct du pseudo spcifi dans l'un, et le pseudo d'une
 *          num spcifie dans l'autre.
 *     csntc(struct nickinfo *nick, char *message, ...)
 *          envoyer une notice avec le CS
 *    Pour plus d'outils, voir outils.c.
 */

void sig_handler (int crap)
{
	want_to_quit = 1;
	signal(SIGINT, &sig_handler);
	signal(SIGTERM,&sig_handler);
}

void sig_reload (int crap)
{
	signal(SIGHUP, &sig_reload);
}

void sig_alarm(int crap)
{
	unsigned long current = time(NULL);
	check_kills();
	check_bans();
	check_ignores();
	if((lastcheckaccounts + 7200) < current)
	{
		check_accounts();
		check_chans();
		lastcheckaccounts = current;
	}
	if((lastresetparse + WAIT_DEL_PARSES) < current)
	{
		remove(LOG_PARSES);
		lastresetparse = current;
	}
	if((lastresetlogs + WAIT_DEL_LOGS) < current)
	{
		remove(LOG_CMDS);
		remove(LOG_CHANS);
		lastresetlogs =current;
	}
	alarm(ALARMINTERVAL);
}

int main(int argc, char **argv)
{
	int sock, irc_ip, r, i = 0, restarted = 0;
	aNick *nick, *nt;
	aChan *chan;
	FILE *fuptime;
	char buff[150];
#ifdef N2K
	char *b;
#endif

	if((fuptime = fopen(SCODERZ_PID, "r")))
	{
		while(fgets(buff, 150, fuptime))
		{
			strip_newline(buff);
			sscanf(buff, "%d", &tuptime);
		}
		printf("SCoderZ est dj lanc sur le pid %d.\n", tuptime);
		printf("Si ce n'est pas le cas, supprimez le fichier '%s' et recommancez.\n", SCODERZ_PID);
		fclose(fuptime);
		exit(-1);
	}

	if (load_config(FICHIER_CONF)== (-1))
	{
		printf("Erreur lors de la lecture de la configuration\n");
		exit(-1);
	}

	if((fuptime = fopen("uptime.tmp", "r")))
	{
		while(fgets(buff, 150, fuptime))
		{
			strip_newline(buff);
			sscanf(buff, "%d %d %d",&tuptime, &lastresetparse, &lastresetlogs);
		}
		fclose(fuptime);
		remove("uptime.tmp");
		restarted = 1;
	}
	load_cmds();
	load_chans();
	load_users();
	load_badchans();
#ifdef USE_WELCOMESERV
	load_welcome();
#endif
	load_badnicks();
#ifdef HAVE_VOTE
	load_votes();
#endif
	STS = time(NULL);
	signal(SIGPIPE, SIG_IGN);
	signal(SIGHUP, &sig_reload);
	signal(SIGINT, &sig_handler);
	signal(SIGTERM,&sig_handler);
	signal(SIGALRM, &sig_alarm);

	r=fork();

	if (r == -1)
	{
		printf("Impossible d'tablir une connexion.");
		exit(-1);
	}

	if (r > 0)
	{
		if(!restarted) printf("Services CoderZ %s  Inter System\n", SPVERSION);
		if(premiere)
		{
			printf("Lorsque les services seront sur votre rseau IRC,\n");
			printf("tapez : /%s hello <username> <email> <pass>\n", cs.nick);
		}
		exit(0);
	}

#ifdef N2K
    b=make_servnum(bot.num);
    strcpy(bot.servnum, b);
#ifdef USE_WELCOMESERV
    strcpy(ws.num, b);
    strcat(ws.num, "ANS");
#endif
    strcpy(cs.num, b);
	strcat(cs.num, "ACS");

#else
    strcpy(bot.servnum, make_servnum(bot.num));
	strcpy(cs.num, make_servnum(bot.num));
	strcat(cs.num, "CS");
#ifdef USE_WELCOMESERV
	strcpy(ws.num, make_servnum(bot.num));
	strcat(ws.num, "NS");
#endif
#endif

	if(!premiere)
	{
		for (; cs_cmds[i].cmd[0] != '\0'; i++)
		{
			if (!strcasecmp(cs_cmds[i].cmd,"hello"))
				cs_cmds[i].disable = 1;
			if(cs_cmds[i].realcmd[0] == '\0') strcpy(cs_cmds[i].realcmd, cs_cmds[i].cmd);
		}
	}

	if(!(fuptime = fopen(SCODERZ_PID, "w"))) printf("Impossible d'crire le ficier PID...\n");
	else
	{
		fprintf(fuptime, "%d\n", getpid());
		fclose(fuptime);
	}

	while (want_to_quit==0)
	{
		irc_ip = resolve(bot.ip);
		if ((irc_ip == 0) || (irc_ip == -1))
		{
			sock = -1;
		} else sock = init_bot(irc_ip, bot.port);
		bot.ipnum = irc_ip;

		if (sock<0) putlog(LOG_PARSES, "Connection choue\n");
		else run_bot(sock);

		if (want_to_quit==1) close(sock);
		else
		{
			putlog(LOG_PARSES, "Dconnect par le serveur\n");

			for(nick = nickhead;nick;nick = nt)
			{
				nt=nick->next;
				if(nick->user) nick->user->n = NULL;
				del_nickinfo(nick);
			}

			for(chan = chanhead;chan;chan = chan->next) DelCJoined(chan);
			close(sock);
			sleep(WAIT_CONNECT);
		}
	}
	putlog(LOG_PARSES, "Fermeture normale du programme\n");
	return (0);
}
